-- Praktikum digitaler Schaltungsentwurf I, Aufgabe 7
-- Datei: RS232.vhdl
--
-- VHDL-Beschreibung einer Seriellen Schnittstelle
-- Empfaenger vorgegeben
--
-- G. Kemnitz
-- 11.05.2015, C. Giesemann - modifiziert NEXYS 3
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity RS232 is
    port(
        GCLK : in std_logic;
        RXD : in std_logic;
        TXD : out std_logic;
        LD : out std_logic_vector(7 downto 0);
        LA : out std_logic_vector(1 downto 0);
        Fehler : out std_logic
    );
end entity;

architecture synth of RS232 is
    -- Internes txd Signal
    -- (da der Port txd ein Ausgabeport ist, kann er nicht gelesen werden)
    signal txd_out : std_logic := '0';

    -- Teilerkostante zur Erzeugung der 16-fachen Baudrate
    -- Formel: 100 MHz / (Baudrate * 16) = 2 * cTeiler
    constant cTeiler : integer := 334;

    -- Taktsignal fuer 16-fache Baudrate
    signal clk_16x_baud : std_logic := '0';
    
    -- 2 Bit Schieberegister fuer Empfangsdaten
    signal rxd_del : std_logic_vector(1 downto 0) := (others => '0');
  
    -- Zustand des Empfangsautomaten
    signal r_state : integer range 0 to 15 := 0;
  
    -- Empfangspuffer
    signal Q : std_logic_vector(7 downto 0) := (others => '0');

begin

    -- Noch kein Sender => Echofunktion
    -- Zeile auskommentieren oder loeschen wenn der Sendeautomat eingebaut wird
    txd_out <= RXD;

    -- Instanz des Sendeautomaten erzeugen
--    sender_inst : entity work.RS232_sender
--        port map(
--            clk_16x_baud => clk_16x_baud,
--            txd          => txd_out,
--            data         => sw,
--            req          => ...,
--            grant        => ...
--        );

    -- Taktteiler zur Erzeugung der 16-fachen Baudrate
    process(GCLK)
        variable counter : integer range 0 to cTeiler - 1 := cTeiler - 1;
    begin
        if rising_edge(GCLK) then
            if counter = 0 then
                clk_16x_baud <= not clk_16x_baud;
                counter := cTeiler - 1;
            else
                counter := counter - 1;
            end if;
        end if;
    end process;

    -- Empfangsautomat
    process(clk_16x_baud)
        variable sample_counter : integer range 0 to 15 := 7;
        variable busy : boolean := false;
    begin
        if rising_edge(clk_16x_baud) then
            -- 2 Bit Schieberegister mit den empfangenen Daten fuellen,
            -- um Pegelwechsel von 1 auf 0 erkennen zu koennen
            rxd_del <= rxd_del(0) & RXD;


            -- Fallende Flanke detektiert
            if rxd_del = "10" and not busy then
                busy := true;
                sample_counter := 7;
            elsif busy then
                -- Zaehler zum Finden des idealen Abtastzeitpunktes
                -- (dieser liegt in der Mitte des Bitintervalls)
                if sample_counter = 0 then
                    sample_counter := 15;
                else
                    sample_counter := sample_counter - 1;
                end if;
            end if;
            
            if busy and sample_counter = 0 then
                r_state <= r_state + 1;
                
                -- Zustandsuebergaenge
                case r_state is
                    -- Startbit
                    when 0 =>
                        -- Startbit empfangen
                        
                    -- 8 Datenbits
                    when 1 to 8 =>
                        Q <= RXD & Q(7 downto 1);
                        
                    -- Paritätsbit
                    when 9 =>
                        -- Paritaet berechnen und auf Fehler-LED ausgeben
                        -- fehler <= ..
                        
                    -- Stoppbit
                    when others =>
                        r_state <= 0;
                        busy := false;
                end case;
            end if;
        end if;
    end process;

    -- Ausgabe des Schieberegisterzustandes
    LD(7 downto 0) <= Q;
    
    -- Ausgabe von rxd und txd an die LA Pins
    LA <= RXD & txd_out;
    
    -- Internes txd Signal an Port zuweisen
    TXD <= txd_out;

end architecture;
