-- Praktikum digitaler Schaltungsentwurf I, Aufgabe 7
-- Datei: RS232_tb.vhdl
--
-- Testrahmen zur Seriellen Schnittstelle
--
-- G. Kemnitz
-- 11.05.2015, C. Giesemann - modifiziert NEXYS 3
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity RS232_tb is
end entity;

architecture sim of RS232_tb is

    constant sim_time : time := 2 ms;
    constant clk_period : time := 650 ns;

    signal clk_16x_baud : std_logic := '0';
    signal txd, req, grant : std_logic := '0';
    signal data : std_logic_vector(7 downto 0) := x"00";
    
begin

    -- Takt mit der 16-fachen Baudrate erzeugen
    process
    begin
        while NOW < sim_time loop
            wait for clk_period / 2;
            clk_16x_baud <= '1';
            wait for clk_period / 2;
            clk_16x_baud <= '0';
        end loop;
        wait;
    end process;

    -- Instanz des Sendeautomaten erzeugen
    sender_inst : entity work.RS232_sender
        port map(
            clk_16x_baud => clk_16x_baud,
            txd          => txd,
            data         => data,
            req          => req,
            grant        => grant
        );

    process
    begin
        -- Sendepause
        wait for 16 * clk_period;

        -- Sende 10 verschiedene Zeichen
        for i in 0 to 9 loop
            data <= std_logic_vector(to_unsigned(i, 8) + x"30");

            -- Setze Anforderungssignal
            req <= '1';
            -- Warte auf Bestätigung
            wait until rising_edge(clk_16x_baud) and grant = '1';
            -- Anforderungssignal zurücknehmen
            req <= '0';
            -- Warte auf Bestätigung
            wait until rising_edge(clk_16x_baud) and grant = '0';

            -- Sendepause
            wait for 128 * clk_period;
        end loop;

        wait;
    end process;

end architecture;
